/*
**********************************************************************************
*  2014 Microchip Technology Inc. and its subsidiaries.
* You may use this software and any derivatives exclusively with
* Microchip products.
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".
* NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE,
* INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY,
* AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
* TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL
* CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF
* FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
* MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
* OF THESE TERMS.
**********************************************************************************
*  GpioTest.cpp
*   This file gives the sample code/ test code for GPIO Interface
*
**********************************************************************************
*  $Revision: #1 $  $DateTime: 2014/03/17 18:04:00 $  $    $
*  Description: initial version
**********************************************************************************
* $File:  
*/
#include "stdafx.h"
#include "GpioTest.h"

#define GPIO32_OUT_ADDR 0x0834
#define GPIO64_OUT_ADDR 0x0934

/*
* RunGPIOTests:
*	This sample code opens handle to the USB2530 device
*	reads state of the specified GPIO pin and also
*	sets the state of the specified GPIO pin to the given
*	state.
*
*	Hardware configuration:
*		Tested using USB2534 QFN48 eval board rev 1.3
*		Refer datasheet/board manual for more information 
*		on GPIO pin number
*/
MCHP_USB_ERROR RunGPIOTests()
{
	HANDLE hndl; 
	int PinNumber = 4;
	int State = 0;
	MCHP_USB_ERROR error = MCHP_USB_ERROR::MCHP_Error_Success;
	char ch;

	printf("********************************************************************\n");
	printf("*       GPIO TEST :  READ and WRITE TO GPIO PIN                    *\n");
	printf("********************************************************************\n");

	/*
	* 1. Open handle to the connected USB2530 device 
	*/
	hndl = MchpUsbOpen (0x0424, 0x2530);
	if(NULL == hndl) {
		printf("\nFailed open Usb Handle\n");
		return MCHP_Error_Invalid_Device_Handle;
	}
	
	/*
	* 2. Read state of the specified GPIO pin using
	*	 MchpUsbInterface API
	*/
	printf("\nEnter pin number to read\n");
	scanf("%d", &PinNumber);
		
	error = GetGPIOState(hndl, PinNumber);
	if(MCHP_Error_Success == error) {
		printf("GPIO Read Test Passed\n");
	} else {
		printf("GPIO Read Test Failed with error code %d\n", error);
	}

	/*
	* 3. Set state of the specified GPIO pin using
	*	 MchpUsbInterface API
	*/
	printf("\nEnter Pin number to write\n"); 
	scanf("%d", &PinNumber);
	printf("Enter the value to write\n");
	scanf("%d", &State);
	error = SetGPIOState(hndl, PinNumber, State);
	if(MCHP_Error_Success == error) {
		printf("GPIO Write Test Passed\n");
	} else {
		printf("GPIO Write Test Failed with error code %d\n", error);
	}

	/*
	* 4. Close handle to the usb device
	*/
	error = MchpUsbClose(hndl);

	return error;
}

/*
*	This sample explains reading the state of the specified
*	GPIO pin "PIONumber"
*/
MCHP_USB_ERROR GetGPIOState(HANDLE hndl, int PIONumber)
{
	MCHP_USB_ERROR result = MCHP_Error_Success;
	int PinState = 0;

	/*
	*	2. Mchp API which retrives the state of the pin 
	*/
	result = MchpUsbGpioGet(hndl, PIONumber, &PinState);

	if(MCHP_Error_Success == result) {
		printf("State of GPIO %d is %d\n", PIONumber, PinState);
	} else {
		printf("GPIO state read failed with error code : %d\n", result);
	}

	return result;
}

/*
*	This sample explains setting the state of the specified
*	GPIO pin "PIONumber"
*/
MCHP_USB_ERROR SetGPIOState(HANDLE hndl, int PIONumber, int State)
{
	MCHP_USB_ERROR result = MCHP_Error_Success;
	int xdataAddr;
	BYTE data;
	int actualLen = 0;
	DWORD dwMask = 0;

	/*
	/* 3. MchpUSB2530 API which sets the GPIO pin to specified state 0/1
	*/
	result = MchpUsbGpioSet(hndl, PIONumber, State);

	/* 
	*  Verification : Read XDATA register using SDKConfig Tool/ directly calling
	*  XDATA APIs to verify the state of corresponding to the GPIO 
	*  (refer PAS for GPIO XDATA MAP)
	*/ 
		
	return result;
}
