/*
**********************************************************************************
*  2014 Microchip Technology Inc. and its subsidiaries.
* You may use this software and any derivatives exclusively with
* Microchip products.
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS".
* NO WARRANTIES, WHETHER EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE,
* INCLUDING ANY IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY,
* AND FITNESS FOR A PARTICULAR PURPOSE, OR ITS INTERACTION WITH MICROCHIP
* PRODUCTS, COMBINATION WITH ANY OTHER PRODUCTS, OR USE IN ANY APPLICATION.
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE.
* TO THE FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL
* CLAIMS IN ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF
* FEES, IF ANY, THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
* MICROCHIP PROVIDES THIS SOFTWARE CONDITIONALLY UPON YOUR ACCEPTANCE
* OF THESE TERMS.
**********************************************************************************
*  UartTest.cpp
*   This file gives the sample code/ test code for Uart Interface
*
**********************************************************************************
*  $Revision: #1 $  $DateTime: 2014/03/17 18:04:00 $  $    $
*  Description: initial version
**********************************************************************************
* $File:  
*/
#include "stdafx.h"
#include "UartTest.h"

int CreateBuffer(unsigned char **buff, int size);
void DestroyBuffer(unsigned char *buff);

/*
* RunUartTests:
*	This sample code expains setting baud rate and 
*	transfer/receive data through serial port interface
*
*	Hardware configuration:
*		Tested using USB2534 eval board QFN48 pin 
*		Used FTDI device and hyperterminal/teraterm for COM communication
*		Refer datasheet/board manual for more information
*		on board setting for UART Enable
*/
MCHP_USB_ERROR RunUartTests()
{
	HANDLE hndl; 
	UCHAR  ucData[64];
	INT ActualBytesRead=0; 
	INT ActualBytesWritten = 0;
	MCHP_USB_ERROR error = MCHP_USB_ERROR::MCHP_Error_Success;
	char ch;
	
	printf("********************************************************************\n");
	printf("*       UART TEST												   *\n");
	printf("********************************************************************\n");
	
	/*
	* 1. Open handle to the connected USB2530 device 
	*/
	hndl = MchpUsbOpen (0x0424, 0x2530);
	if(NULL == hndl) {
		printf("\nFailed open Usb Handle\n");
		return MCHP_Error_Invalid_Device_Handle;
	}
	
	
	do {
		/*
		* 2. Set baud rate to 9600
		*/
		printf("\nStep 1. Setting Baud Rate to 9600\n");
		error = MchpUsbUartSetConfig(hndl, 9600, 0, 0, 0);
	
		if(error != MCHP_Error_Success) {
			printf("UART Set Config Failed. Error %d\n", error);
			break;
		}
		
		/*
		* 3. Uart transmit 100 bytes using MchpUsb2530 API
		*	 and receive thru COM terminal
		*/
		printf("\nStep 2. Uart transmitting 100 bytes repeating data 0x00 to 0x63 at baud rate 9600\n");
		printf("Set baud rate of receiver to 9600 and press a key\n");
		scanf("%c", &ch);
		error = UartTransmitData(hndl);
		if(error != MCHP_Error_Success)	{
			printf("UART Transmit Failed. Error %d\n", error);
			break;
		}
		
		/*
		* 4. Uart transmit 100 bytes from COM terminal and 
		*	 receive synchronously using MchpUsb2530 API
		*/
		printf("\nStep 3. Uart Waiting to Receive 100 bytes at baud rate 9600...\n");
		error = UartReceiveDataFromTerm(hndl);
		if(error != MCHP_Error_Success)	{
			printf("UART Receive Failed. Error %d\n", error);
			break;
		}
		
		/*
		* 5. Set baud rate to 19200
		*/
		printf("\nStep 4. Setting Baud Rate to 19200\n");
		error = MchpUsbUartSetConfig(hndl, 19200, 0, 0, 0);	
		if(error != MCHP_Error_Success) {
			printf("UART Set Config Failed. Error %d\n", error);
			break;
		}
		
		/*
		* 6. Uart transmit 100 bytes using MchpUsb2530 API
		*	 and receive thru COM terminal
		*/
		printf("\nStep 5. Uart transmitting 100 bytes repeating data 0x00 to 0x63 at baud rate 19200\n");
		printf("Set baud rate of receiver to 19200 and press a key\n");
		scanf("%d", &ch);
		error = UartTransmitData(hndl);
		if(error != MCHP_Error_Success)	{
			printf("UART Transmit Failed. Error %d\n", error);
			break;
		}

		/*
		* 7. Uart transmit 100 bytes from COM terminal and 
		*	 receive synchronously using MchpUsb2530 API
		*/
		printf("\nStep 6. Uart Waiting to Receive 100 bytes at baud rate 19200...\n");
		error = UartReceiveDataFromTerm(hndl);
		if(error != MCHP_Error_Success)	{
			printf("UART Receive Failed. Error %d\n", error);
			break;
		}
	}while (FALSE);

	/*
	* 8. Close handle to the usb device
	*/
	error = MchpUsbClose (hndl);

	return error;
}

/*
* UART transmit 100 bytes 
*/
MCHP_USB_ERROR UartTransmitData(HANDLE hndl)
{
	unsigned char *ucData = NULL;

	if(CreateBuffer(&ucData, 100))
		return MCHP_Error_Undefined;

	MCHP_USB_ERROR error = MCHP_Error_Success;
	int transmitSize;
	
	transmitSize = sizeof(ucData);

	error = MchpUsbUartWrite(hndl, 100, ucData);

	if(ucData != NULL)
		DestroyBuffer(ucData);

	return error;
}

/*
*	UART Receive 100 bytes synchronously
*/
MCHP_USB_ERROR UartReceiveDataFromTerm(HANDLE hndl)
{
	unsigned char ucInput[120] = {0};
	MCHP_USB_ERROR error = MCHP_Error_Success;
	int receiveSize;

	receiveSize = 100;
	error = MchpUsbUartRead(hndl, receiveSize, ucInput);

	printf("%s\n", ucInput);

	return error;
}

int CreateBuffer(unsigned char **buff, int size)
{
	int i = 0;
	unsigned char val = 0;
	
	if(size > 512) {
		printf("use size upto 512 bytes\n");
	}

	*buff = (unsigned char *)malloc(size);
	if(buff == NULL) {
		printf("Buffer allocation failed\n");
		return -1;
	}

	memset(*buff, 0, size);

	while(i < size) {
		(*buff)[i++] = val++;
		if(val == 0xff) {
			val = 0;
		}
	}

	return 0;
}

void DestroyBuffer(unsigned char *buff)
{
	if(buff != NULL) {
		free(buff);
	}
	return;
}